package net.knarcraft.questmobspawns.listener;

import me.blackvein.quests.QuestsAPI;
import me.blackvein.quests.player.IQuester;
import me.blackvein.quests.quests.IQuest;
import me.blackvein.quests.quests.IStage;
import net.knarcraft.questmobspawns.util.QuestMobHelper;
import org.bukkit.Location;
import org.bukkit.entity.Entity;
import org.bukkit.entity.Player;
import org.bukkit.event.Cancellable;
import org.bukkit.event.EventHandler;
import org.bukkit.event.EventPriority;
import org.bukkit.event.Listener;
import org.bukkit.event.entity.EntityBreakDoorEvent;
import org.bukkit.event.entity.EntityChangeBlockEvent;
import org.bukkit.event.entity.EntityDamageByEntityEvent;
import org.bukkit.event.entity.EntityExplodeEvent;
import org.bukkit.event.entity.EntityInteractEvent;
import org.bukkit.event.entity.EntityTargetLivingEntityEvent;

public class MobBlockDamageListener implements Listener {

    private final QuestsAPI questsAPI;

    public MobBlockDamageListener(QuestsAPI questsAPI) {
        this.questsAPI = questsAPI;
    }

    @EventHandler(priority = EventPriority.HIGHEST)
    public void onExplode(EntityExplodeEvent event) {
        if (QuestMobHelper.isSpawnedMob(event.getEntity())) {
            event.blockList().clear();
            event.setCancelled(false);
        }
    }

    @EventHandler(priority = EventPriority.HIGHEST, ignoreCancelled = true)
    public void onBlockChange(EntityChangeBlockEvent event) {
        cancelIfQuestMob(event, event.getEntity());
    }

    @EventHandler
    public void onDoorBreak(EntityBreakDoorEvent event) {
        cancelIfQuestMob(event, event.getEntity());
    }

    @EventHandler
    public void onEntityTarget(EntityTargetLivingEntityEvent event) {
        Entity entity = event.getEntity();
        if (!QuestMobHelper.isSpawnedMob(entity) || !(event.getTarget() instanceof Player player)) {
            return;
        }

        if (!hasQuest(entity, player)) {
            event.setCancelled(true);
        } else {
            // Cancel the targeting if the player is outside the kill-zone
            int[] killLocationData = QuestMobHelper.getKillZoneLocation(entity);
            Location killLocation = new Location(entity.getWorld(), killLocationData[0], killLocationData[1],
                    killLocationData[2]);
            event.setCancelled(QuestMobHelper.get2dDistance(player.getLocation(), killLocation) > killLocationData[3]);
        }
    }
    
    @EventHandler
    public void onTrample(EntityInteractEvent event) {
        cancelIfQuestMob(event, event.getEntity());
    }

    @EventHandler(priority = EventPriority.HIGHEST)
    public void onEntityDamage(EntityDamageByEntityEvent event) {
        if (!QuestMobHelper.isSpawnedMob(event.getDamager())) {
            return;
        }

        Entity damagingEntity = event.getDamager();
        if (event.getEntity() instanceof Player player) {
            event.setCancelled(!hasQuest(damagingEntity, player));
        } else {
            event.setCancelled(true);
        }
    }

    /**
     * Checks whether the given player has the quest the given entity was spawned for
     * 
     * @param entity <p>The entity with stored quest data</p>
     * @param player <p>A player</p>
     * @return <p>True if the player has the quest stored in the entity</p>
     */
    private Boolean hasQuest(Entity entity, Player player) {
        IQuester questPlayer = questsAPI.getQuester(player.getUniqueId());
        IQuest quest = getQuest(QuestMobHelper.getQuestId(entity));
        if (quest == null) {
            // Remove the entity if the quest it belongs to no longer exists
            entity.remove();
            return false;
        }
        IStage stage = quest.getStage(QuestMobHelper.getStageId(entity));
        int index = QuestMobHelper.getIndex(entity);
        return questPlayer.getQuestData().containsKey(quest) && questPlayer.getCurrentStage(quest) == stage &&
                questPlayer.getQuestData(quest).getMobNumKilled().get(index) < stage.getMobNumToKill().get(index);
    }

    private IQuest getQuest(String questId) {
        for (IQuest quest : questsAPI.getLoadedQuests()) {
            if (quest.getId().equalsIgnoreCase(questId)) {
                return quest;
            }
        }
        return null;
    }
    
    private void cancelIfQuestMob(Cancellable cancellable, Entity entity) {
        if (QuestMobHelper.isSpawnedMob(entity)) {
            cancellable.setCancelled(true);
        }
    }

}
